#!/usr/bin/env python2
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
#"THE BEER-WARE LICENSE" (Revision 42):
#Simon Chabot <chabotsi@unice.fr> wrote this file. As long as you retain this
#notice you can do whatever you want with this stuff. If we meet some day, and
#you think this stuff is worth it, you can buy me a beer in return.
#-------------------------------------------------------------------------------


from __future__ import (unicode_literals, absolute_import, division,
                        print_function)

from math import pi
from numpy import log10, exp, where, sin, log, sqrt, cos, arccos, sin
from scipy.optimize import bisect

from constants import (GRAVITY, EARTH_RADIUS, P_X, R_X, H_SCALE, H_ATM, C_D,
                       F_PANCAKE, SIGMA)

class Asteroid(object):
    r""" Defines all the data needed to the calculations """

    def __init__(self):

        self.asteroid_density = None
        self.asteroid_diameter = None
        self.asteroid_velocity = None
        self.asteroid_angle = None

        self.ground_density = None

        self.water_depth = None
        self.bottom = None

        self.crater_complex = False
        self.crater_diameter_tr = None
        self.crater_diameter_final = None

    @property
    def is_over_the_ocean(self):
        try:
            return (self.water_depth[0] > 0)
        except TypeError:
            return False

    @property
    def kinematic_energy(self):
        r""" the asteroid’s kinematic energy """

        rho = self.asteroid_density
        L = self.asteroid_diameter
        v = self.asteroid_velocity

        return pi/12*rho*L**3*v**2

    @property
    def kinematic_energy_MT(self):
        r""" the asteroid’s kinematic energy in MegaTons"""
        return self.kinematic_energy/4.18e15

    def atm_density(self, z):
        return exp(-z/H_SCALE)

    def velocity(self, z=0):
        v = self.asteroid_velocity
        L = self.asteroid_diameter
        theta = self.asteroid_angle
        rho_atm_z = self.atm_density(z)
        rho_i = self.asteroid_density
        return v*exp(-3*rho_atm_z*C_D*H_SCALE/(4*rho_i * L * sin(theta)))

    @property
    def z_rupture(self):
        y_i = 10**(2.107 + 0.0624*sqrt(self.asteroid_density))
        f = lambda z: y_i - self.atm_density(z)*self.velocity(z)**2 #
        try:
            return bisect(f, 0, H_ATM, xtol=H_SCALE/10)
        except ValueError: #f(0) and f(H_ATM) have the same sign
            return 0

    @property
    def z_airburst(self):
        z_r = self.z_rupture

        if z_r == 0:
            return 0

        v = self.asteroid_velocity
        L = self.asteroid_diameter
        rho = self.asteroid_density
        theta = self.asteroid_angle

        l = L*sin(theta)*sqrt(rho/(C_D*self.atm_density(z_r)))

        return z_r - 2*H_SCALE*log(1 + l/(2*H_SCALE)*sqrt(F_PANCAKE**2 - 1))

    def explosion_damages(self, distance): #Vincent’s stuff
        r""" knowning where the asteroid exploses, here we compute the damage
        received at a distance `distance` """
        if self.z_airburst > 0: #has explosed in the air
            return self._airburst(distance)
        else:
            return self._airblast(distance)

    def explosion_radius(self, P_damages=7000): #Vincent
        r""" Compute the radius where the explosion pressure is above
        `P_damages`. Here P_damages=7000 means all the windows explose."""
        r = 100
        while self.explosion_damages(r) < P_damages:
            r += 100

        return r

    def _airblast(self, distance):
        r1 = distance / self.kinematic_energy**(1/3)
        return P_X*R_X/(4 * r1)*(1 + 3*(R_X/r1)**(1/3))

    def _airburst(self, distance):
        r1 = distance / self.kinematic_energy**(1/3)
        p0 = exp(3.14e11 * self._z_rupture**(-2.16))
        beta = 34.87 * self._z_rupture**(-1.73)
        return p0*exp(-beta*r1)

    def tsunami_amplitude(self):
        r""" Return the tsunami amplitude (Green law) at a given distance from
        the impact with a given deepness """

        if not self.is_over_the_ocean:
            return

        H = self.water_depth[0]
        h = self.water_depth
        ek = self.kinematic_energy

        amplitude = H * ek**(0.5) * h**(-0.25)

        #cheating part
        e = int(log10(amplitude[0]))
        coef = 10**(1-e)
        print(coef)
        return amplitude*coef # / amplitude[0] #TODO de-normalized it

    def sismic_magnitude(self, distance=0):
        k = self.kinematic_energy

        d_km = distance/1e3

        m = 0.67*log10(k) - 5.87
        m = where(d_km < 60, m - 0.0238*d_km, m)
        m = where((60 <= d_km) & (d_km < 700), m - 0.0048*d_km - 1.1644, m)
        m = where(700 <= d_km, m - 1.66*log10(distance/EARTH_RADIUS) - 6.399, m)

        try:
            return float(m)
        except TypeError: #distance is an array, return an array
            return m

    def crater(self):
        rho_ast = self.asteroid_density
        rho_earth = self.ground_density
        ast_dia = self.asteroid_diameter
        v = self.asteroid_velocity
        theta = self.asteroid_angle

        c = 1.161 if not self.is_over_the_ocean else 1.365
        simple_max_diameter = 3200

        self.crater_diameter_tr = c * (rho_ast/rho_earth)**(1./3.) \
                                    * (ast_dia**0.78) \
                                    * (v**0.44) \
                                    * (GRAVITY**-0.22) \
                                    * sin(theta)**(1./3.)
        if self.crater_diameter_tr < simple_max_diameter:
            self.crater_complex = False
            self.crater_diameter_final = 1.25*self.crater_diameter_tr
        else:
            self.crater_complex = True
            self.crater_diameter_final = 1.17*self.crater_diameter_tr**1.13 \
                                         /(simple_max_diameter**0.13)

    def ejecta_thickness(self, distance):
        return (self.crater_diameter_tr)**4/(112*distance**3)

    def ejecta_size(self, distance):
        dfinal = self.crater_diameter_final/1e3 #km
        return 2400*(dfinal/2)**(-1.62)*(dfinal/(2*distance/1e3))**2.65

    @property
    def fireball_radius(self):
        return 2e-3*self.kinematic_energy**(1/3)

    def thermic(self, r_target=0, temperature=300):
        r""" return MJ/m²"""
        ek = self.kinematic_energy
        r_ball = self.fireball_radius
        temperature_kelvin = temperature + 273.15

        delta = r_target / EARTH_RADIUS
        h = (1-cos(delta))*EARTH_RADIUS
        alpha = arccos(h/r_ball)
        f = (2/pi)*(alpha-(h/r_ball)*sin(alpha))

        phi = (3e-3*ek)/(2*pi*r_target**2)
        irradiation_duration = (f*3e-3*ek)/((2*pi*r_target**2)*(SIGMA*temperature_kelvin**4));

        return f*phi/1e6, irradiation_duration
