#!/usr/bin/env python2
# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
#"THE BEER-WARE LICENSE" (Revision 42):
#Simon Chabot <chabotsi@unice.fr> wrote this file. As long as you retain this
#notice you can do whatever you want with this stuff. If we meet some day, and
#you think this stuff is worth it, you can buy me a beer in return.
#-------------------------------------------------------------------------------

from __future__ import (unicode_literals, absolute_import, division,
                        print_function)

from PySide import QtGui
from PySide.QtCore import Slot, Qt

import os
os.environ["QT_API"] = "pyside"

import matplotlib
matplotlib.use('Qt4Agg')
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
matplotlib.rcParams['backend.qt4']='PySide'
import matplotlib.pyplot as plt
import numpy as np
from scipy.interpolate import splrep, splev
from math import pi

from impactlocationwindow import ImpactLocationWindow
from asteroid import Asteroid
from sampledisplaywindow  import DensityDisplayWindow, DiameterDisplayWindow
from mainwindow_ui import Ui_MainWindow

from constants import (MAGNITUDE_MIN, H_SCALE, H_ATM, WORLP_MAX_RESOLUTION,
                       TSUNAMI_MIN_DEEP, PHI_IGNITION_1MT, MIN_TEMPERATURE)

class AsteroidMainWindow(QtGui.QMainWindow):
    def __init__(self):
        super(AsteroidMainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.distance = np.linspace(0, 500, 1000) #typical distance used for calcultations, 500km
        self._init_plot()
        self._loadimg()

        self.impactLocationWindow = ImpactLocationWindow()
        self.densitychooserwindow = DensityDisplayWindow()
        self.diameterchooserwindow = DiameterDisplayWindow()

        self._make_connections()
        self.data = Asteroid()
        self.show()

    def _loadimg(self):
        self.img = {}
        self.img['tr_crater'] = QtGui.QPixmap('img/tr_crater.png')
        self.img['complex_crater'] = QtGui.QPixmap('img/complex_crater.png')
        self.img['simple_crater'] = QtGui.QPixmap('img/simple_crater.png')
        img = self.img['tr_crater'].scaled(self.ui.tr_crater_img.size(), Qt.KeepAspectRatio)
        self.ui.tr_crater_img.setPixmap(img)

    def _init_plot(self):
        self.figures = {}
        self.canvas = {}

        for key in ('atm', 'sismic', 'tsunami', 'explosion', 'thermic', 'ejecta'):
            self.figures[key] = plt.figure()
            self.canvas[key] = FigureCanvas(self.figures[key])
            getattr(self.ui, '{}_layout'.format(key)).addWidget(self.canvas[key])

    def _make_connections(self):
        """ Make connection between mainwindow’s objets and slots """

        self.ui.densitySpinBox.editingFinished.connect(self.update_results)
        self.ui.diameterDoubleSpinBox.editingFinished.connect(self.update_results)
        self.ui.velocityDoubleSpinBox.editingFinished.connect(self.update_results)
        self.ui.angleSpinBox.editingFinished.connect(self.update_results)
        self.ui.grounddensitySpinBox.editingFinished.connect(self.update_results)
        self.ui.depthSpinBox.editingFinished.connect(self.update_results)


        self.ui.asteroiddensitychooserPushbutton.clicked.connect(self.densitychooserwindow.show)
        self.ui.diameter_example_pushButton.clicked.connect(self.diameterchooserwindow.show)
        self.densitychooserwindow.sample_chosen.connect(self.ui.densitySpinBox.setValue)
        self.densitychooserwindow.sample_chosen.connect(self.update_results)
        self.diameterchooserwindow.sample_chosen.connect(self.ui.diameterDoubleSpinBox.setValue)
        self.diameterchooserwindow.sample_chosen.connect(self.update_results)
        self.ui.impactLocationPushButton.clicked.connect(self.impactLocationWindow.show)
        self.impactLocationWindow.location_chosen.connect(self.update_water_depth)
        self.ui.tabWidget.currentChanged.connect(self.tab_changed)

    @Slot()
    def update_data(self):
        """ Update all the data of the ImpactData object """

        self.data.asteroid_density = self.ui.densitySpinBox.value()
        self.data.asteroid_diameter = self.ui.diameterDoubleSpinBox.value() * 1e3 #km
        self.data.asteroid_velocity = self.ui.velocityDoubleSpinBox.value() * 1e3 #km
        self.data.asteroid_angle = self.ui.angleSpinBox.value() * pi/180 #radians
        self.data.ground_density = self.ui.grounddensitySpinBox.value()


    @Slot(list)
    def update_water_depth(self, colors):
        r""" Update water depth value according to the color of the clicked pixel
        """
        water_depth = []
        for color in colors:
            c = color.red()
            depth = 5120 - c*40 if c < 128 else 0
            water_depth.append(depth)
            if depth == 0:
                break
        #interpolate the water_depth
        size = len(water_depth)
        if size == 1: #ground has been picked twice
            self.ui.depthSpinBox.setValue(0)
            self.data.water_depth = None
            self.data.bottom = None
        else:
            seabottom = np.linspace(0, size*WORLP_MAX_RESOLUTION, size)
            tck = splrep(seabottom, water_depth)

            seabottom = np.linspace(0, size*WORLP_MAX_RESOLUTION, 10*size) #over sample it 10 times
            water_depth = splev(seabottom, tck)

            #take only the positive part
            water_depth = water_depth[water_depth > TSUNAMI_MIN_DEEP]
            seabottom = seabottom[water_depth > TSUNAMI_MIN_DEEP]

            self.ui.depthSpinBox.setValue(water_depth.max())
            self.data.water_depth = water_depth
            self.data.bottom = seabottom

        self.update_results()

    @Slot()
    def tab_changed(self, index):
        if index == 1:
            self.update_results()
            self._update_detailed_results()

    @Slot()
    def update_results(self):
        self.update_data()
        self._update_simple_results()
        #self._update_detailed_results()

    def _update_simple_results(self):
        """ Update the simulation simple results box """
        kinematic_energy = self.data.kinematic_energy
        magnitude = self.data.sismic_magnitude()
        try:
            amplitude = self.data.tsunami_amplitude()[0]
        except TypeError:
            amplitude = 0


        self.data.crater()
        dcrater = self.data.crater_diameter_final


        self.ui.kineticEnergyLineEdit.setText('{:.3e} J'.format(kinematic_energy))
        self.ui.magnitude_doubleSpinBox.setValue(magnitude)
        self.ui.tsunami_doubleSpinBox.setValue(amplitude)
        self.ui.craterDiameterDoubleSpinBox.setValue(dcrater/1e3)

    def _update_detailed_results(self):
        """ Update the simulation detailed results tab """

        self.clean_plot()
        is_over_the_ocean = self.data.is_over_the_ocean

        order = ('atm', 'explosion', 'crater', 'thermic', 'sismic', 'tsunami', 'ejecta')
        everything = set(order)
        when_over_ground = everything - set(('tsunami', ))
        when_over_ocean = everything - set(('sismic', 'ejecta', 'thermic'))

        for item in order:
            cond = (is_over_the_ocean and item in when_over_ocean) or \
                   (not is_over_the_ocean and item in when_over_ground)
            if cond:
                getattr(self.ui, 'page_{}'.format(item)).show()
                getattr(self, 'make_{}'.format(item))()
            else:
                getattr(self.ui, 'page_{}'.format(item)).hide()

    def clean_plot(self):
        for p in self.figures.itervalues():
            p.clf()

    def make_atm(self):
        z_rupture = self.data.z_rupture
        z_airburst = self.data.z_airburst
        alt_max = z_rupture*3
        altitude = np.arange(0, alt_max, 1000)
        velocity = self.data.velocity(altitude)
        v_min = velocity.min()
        v_max = velocity.max()

        ax = self.figures['atm'].add_subplot(111)
        if 0 < z_rupture and z_rupture < alt_max:
            ax.axvline(x=z_rupture/1e3, color='g', linestyle='--', label='break-up alt.')
        if 0 < z_airburst and z_airburst < alt_max:
            ax.axvline(x=z_airburst/1e3, color='r', linestyle='--', label='airburst alt.')
            ax.plot(altitude[altitude > z_airburst]/1e3,
                    velocity[altitude > z_airburst]/1e3, 'b')
            ax.plot(altitude[altitude < z_airburst]/1e3,
                    velocity[altitude < z_airburst]/1e3, 'b--')
        else:
            ax.plot(altitude/1e3, velocity/1e3, 'b')
        ax.legend(loc='lower left')
        ax.set_ylabel('velocity (km.s⁻¹)')
        ax.set_xlabel('altitude (km)')
        ax.get_yaxis().get_major_formatter().set_useOffset(False)
        ax.invert_xaxis() #make the altitude decays when x increases
        self.canvas['atm'].draw()
        #self.figures['sismic'].tight_layout()
        
        #self.ui.breakpup_altitude_doublespinbox.setValue(self.data.z_rupture)
        #self.ui.airburst_altitude_doubleSpinBox.setValue(self.data.z_airburst)

    def make_sismic(self):
        distance = np.linspace(0, 10000e3, 1000) #over 10 000 km
        magnitude = self.data.sismic_magnitude(distance)
        ax = self.figures['sismic'].add_subplot(111)
        ax.semilogx(distance/1e3, magnitude, 'b')

        #horizontal lines
        if magnitude.max() > 8:
            ax.semilogx(distance/1e3, 8*np.ones_like(distance), 'k--', label='strong damage')
        if magnitude.max() > 6:
            ax.semilogx(distance/1e3, 6*np.ones_like(distance), 'r--', label='moderate damage')
        if magnitude.min() < 3:
            ax.semilogx(distance/1e3, 3*np.ones_like(distance), 'g--', label='felt by all')
        ax.legend()
        ax.set_ylabel('Richter magnitude')
        ax.set_xlabel('Distance from the epicentre (km)')
        ax.set_ylim(max(2, magnitude.min()), magnitude.max())
        self.canvas['sismic'].draw()
        self.figures['sismic'].tight_layout()

    def make_tsunami(self):
        if not self.data.is_over_the_ocean:
            return

        amplitude = self.data.tsunami_amplitude()
        bottom = self.data.bottom

        ax = self.figures['tsunami'].add_subplot(211)
        ax.plot(bottom/1e3, amplitude, 'b')
        ax.set_ylabel('amplitude')
        ax.set_xlabel('Distance from the epicentre (km)')

        ax = self.figures['tsunami'].add_subplot(212)
        ax.plot(bottom/1e3, -self.data.water_depth, 'b')
        ax.set_ylabel('sea bottom')
        ax.set_xlabel('Distance from the epicentre (km)')
        self.canvas['tsunami'].draw()
        self.figures['tsunami'].tight_layout()

    def make_crater(self):
        self.data.crater()
        if self.data.crater_complex:
            img = self.img['complex_crater'].scaled(self.ui.final_crater_img.size(), Qt.KeepAspectRatio)
        else:
            img = self.img['simple_crater'].scaled(self.ui.final_crater_img.size(), Qt.KeepAspectRatio)
        self.ui.final_crater_img.setPixmap(img)
        self.ui.final_diameter_spinbox.setValue(self.data.crater_diameter_final/1e3)
        self.ui.tr_diameter_spinbox.setValue(self.data.crater_diameter_tr/1e3)

    def make_ejecta(self):
        D_tf = self.data.crater_diameter_final
        distance = np.linspace(D_tf, 10000e3, 1000)
        ejecta_thickness = self.data.ejecta_thickness(distance)
        ejecta_size = self.data.ejecta_size(distance)

        ax = self.figures['ejecta'].add_subplot(111)
        ax.loglog(distance/1e3, ejecta_thickness, 'b', label='thickness (m)')
        ax.loglog(distance/1e3, ejecta_size, 'r', label='size (m)')
        ax.set_ylim(1e-2, 10*(max(max(ejecta_thickness), max(ejecta_size))))
        ax.set_xlabel('distance from the epicentre (km)')
        ax.legend()
        self.canvas['ejecta'].draw()
        self.figures['ejecta'].tight_layout()

    def make_thermic(self):
        fireball_r = self.data.fireball_radius
        distance = np.linspace(fireball_r, fireball_r*100, 1000)
        ek_MT = self.data.kinematic_energy_MT

        phi, tau = self.data.thermic(distance, temperature=MIN_TEMPERATURE)
        phi /= (ek_MT**(1/6))

        #dont plot when phi is small
        distance = distance[phi > 0.01]
        tau = tau[phi > 0.01]
        phi = phi[phi > 0.01]

        ax = self.figures['thermic'].add_subplot(111)
        ax.loglog(distance/1e3, phi, label='thermal exposure (MJ/m²) / energy (MT)')
        ax.loglog(distance/1e3, tau/3600,
                  label='duration of temperature > {}°C (hour)'.format(MIN_TEMPERATURE))

        for thing, value in PHI_IGNITION_1MT.iteritems():
            ax.axhline(y=value, linestyle='--', label=thing)

        ax.axvline(x=fireball_r/1e3, color='r', linestyle='--', label='Fireball radius')

        ax.set_xlabel('distance from the epicentre (km)')
        ax.legend()
        self.canvas['thermic'].draw()
        self.figures['thermic'].tight_layout()

    def make_explosion(self):
        pass
